@@ -69,17 +69,14 @@ void DBusControlWrapper::setVolume(int percentage)
 	//  - esken
 	Volume& volP = m_md->playbackVolume();
 	Volume& volC = m_md->captureVolume();
-	volP.setAllVolumes( (percentage * volP.maxVolume()) / 100 );
-	volC.setAllVolumes( (percentage * volP.maxVolume()) / 100 );
+	volP.setAllVolumes( volP.minVolume() + ((percentage * volP.volumeSpan()) / 100) );
+	volC.setAllVolumes( volC.minVolume() + ((percentage * volC.volumeSpan()) / 100) );
 	m_md->mixer()->commitVolumeChange( m_md );
 }
 
 int DBusControlWrapper::volume()
 {
-	Volume& vol = m_md->playbackVolume();
-	return vol.maxVolume()
-		? vol.getAvgVolume( (Volume::ChannelMask)(Volume::MLEFT | Volume::MRIGHT) ) * 100 / vol.maxVolume()
-		: 0;
+	return m_md->playbackVolume().getAvgVolumePercent(Volume::MALL);
 }
 
 void DBusControlWrapper::increaseVolume()
@@ -117,7 +114,9 @@ long DBusControlWrapper::absoluteVolume()
 {
 	// @todo hardcoded
 	Volume& vol = m_md->playbackVolume();
-	return ( vol.getAvgVolume( (Volume::ChannelMask)(Volume::MLEFT | Volume::MRIGHT) ) );
+	qreal avgVol= vol.getAvgVolume( Volume::MMAIN );
+	long avgVolRounded = avgVol <0 ? avgVol-.5 : avgVol+.5;
+	return avgVolRounded;
 }
 
 void DBusControlWrapper::setMute(bool muted)
@@ -149,5 +148,8 @@ bool DBusControlWrapper::isRecordSource()
 
 void DBusControlWrapper::setRecordSource(bool on)
 {
-	m_md->mixer()->setRecordSource( m_md->id(), on );
+	MixDevice* md = m_md->mixer()->getMixdeviceById(m_md->id());
+	if ( md != 0 )
+		md->setRecSource(on);
+	m_md->mixer()->commitVolumeChange( m_md );
 }
