@@ -35,7 +35,6 @@
 #include <qcursor.h>
 #include <QCheckBox>
 #include <QMouseEvent>
-#include <qslider.h>
 #include <QLabel>
 #include <qpixmap.h>
 #include <qwmatrix.h>
@@ -43,6 +42,7 @@
 
 #include "gui/guiprofile.h"
 #include "mdwslider.h"
+#include "volumeslider.h"
 #include "core/mixer.h"
 #include "viewbase.h"
 #include "ksmallslider.h"
@@ -50,13 +50,15 @@
 #include "mdwmoveaction.h"
 
 
-/**
+VolumeSliderExtraData MDWSlider::DummVolumeSliderExtraData;
+bool MDWSlider::debugMe = false;
+ /**
  * MixDeviceWidget that represents a single mix device, inlcuding PopUp, muteLED, ...
  *
  * Used in KMix main window and DockWidget and PanelApplet.
  * It can be configured to include or exclude the captureLED and the muteLED.
  * The direction (horizontal, vertical) can be configured and whether it should
- * be "small"  (uses KSmallSlider instead of QSlider then).
+ * be "small"  (uses KSmallSlider instead of a normal slider widget).
  *
  * Due to the many options, this is the most complicated MixDeviceWidget subclass.
  */
@@ -68,7 +70,7 @@ MDWSlider::MDWSlider(MixDevice* md, bool showMuteLED, bool showCaptureLED,
 	MixDeviceWidget(md,small,orientation,parent,view, par_ctl),
 	m_linked(true),	muteButtonSpacer(0), captureSpacer(0), labelSpacer(0),
 	m_iconLabelSimple(0), m_qcb(0), m_muteText(0),
-	m_extraCaptureLabel( 0 ), m_label( 0 ), /*m_captureLED( 0 ),*/
+	m_label( 0 ), /*m_captureLED( 0 ),*/
 	m_captureCheckbox(0), m_captureText(0), labelSpacing(0),
 	muteButtonSpacing(false), captureLEDSpacing(false), _mdwMoveActions(new KActionCollection(this)), m_moveMenu(0)
 {
@@ -79,41 +81,52 @@ MDWSlider::MDWSlider(MixDevice* md, bool showMuteLED, bool showCaptureLED,
     update();
 }
 
+MDWSlider::~MDWSlider()
+{
+	foreach( QAbstractSlider* slider, m_slidersPlayback)
+	{
+		delete slider;
+	}
+	foreach( QAbstractSlider* slider, m_slidersCapture)
+	{
+		delete slider;
+	}
+}
 
 void MDWSlider::createActions()
 {
     // create actions (on _mdwActions, see MixDeviceWidget)
     KToggleAction *taction = _mdwActions->add<KToggleAction>( "stereo" );
     taction->setText( i18n("&Split Channels") );
-    connect( taction, SIGNAL( triggered(bool) ), SLOT( toggleStereoLinked() ) );
+    connect( taction, SIGNAL(triggered(bool)), SLOT(toggleStereoLinked()) );
 
     KAction *action;
     if ( ! m_mixdevice->mixer()->isDynamic() ) {
         action = _mdwActions->add<KToggleAction>( "hide" );
         action->setText( i18n("&Hide") );
-        connect( action, SIGNAL( triggered(bool) ), SLOT( setDisabled() ) );
+        connect( action, SIGNAL(triggered(bool)), SLOT(setDisabled()) );
     }
 
     if( m_mixdevice->playbackVolume().hasSwitch() ) {
         taction = _mdwActions->add<KToggleAction>( "mute" );
         taction->setText( i18n("&Muted") );
-        connect( taction, SIGNAL( toggled(bool) ), SLOT( toggleMuted() ) );
+        connect( taction, SIGNAL(toggled(bool)), SLOT(toggleMuted()) );
     }
 
     if( m_mixdevice->captureVolume().hasSwitch() ) {
         taction = _mdwActions->add<KToggleAction>( "recsrc" );
         taction->setText( i18n("Set &Record Source") );
-        connect( taction, SIGNAL( toggled(bool) ), SLOT( toggleRecsrc() ) );
+        connect( taction, SIGNAL(toggled(bool)), SLOT(toggleRecsrc()) );
     }
 
     if( m_mixdevice->isMovable() ) {
         m_moveMenu = new KMenu( i18n("Mo&ve"), this);
-        connect( m_moveMenu, SIGNAL(aboutToShow()), SLOT( showMoveMenu()) );
+        connect( m_moveMenu, SIGNAL(aboutToShow()), SLOT(showMoveMenu()) );
     }
 
     action = _mdwActions->addAction( "keys" );
     action->setText( i18n("C&onfigure Shortcuts...") );
-    connect( action, SIGNAL( triggered(bool) ), SLOT( defineKeys() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(defineKeys()) );
 }
 
 void MDWSlider::createShortcutActions()
@@ -130,48 +143,44 @@ void MDWSlider::createShortcutActions()
     QString increaseVolumeName = i18n( "Increase Volume" );
     increaseVolumeName += " - " + mixDevice()->readableName() + ", " + mixDevice()->mixer()->readableName();
     b->setText( increaseVolumeName  );
-    #ifdef __GNUC__
-    #warning GLOBAL SHORTCUTS ARE NOW ASSIGNED TO ALL CONTROLS, as enableGlobalShortcut(), has not been committed
-    #endif
+//     #ifdef __GNUC__
+//     #warning GLOBAL SHORTCUTS ARE NOW ASSIGNED TO ALL CONTROLS, as enableGlobalShortcut(), has not been committed
+//     #endif
     if ( ! mixDevice()->mixer()->isDynamic() ) {
         // virtual / dynamic controls won't get shortcuts
         b->setGlobalShortcut(dummyShortcut);  // -<- enableGlobalShortcut() is not there => use workaround
         //   b->enableGlobalShortcut();
-        connect( b, SIGNAL( triggered(bool) ), SLOT( increaseVolume() ) );
+        connect( b, SIGNAL(triggered(bool)), SLOT(increaseVolume()) );
     }
 
     b = _mdwPopupActions->addAction( QString("Decrease volume %1").arg( actionSuffix ) );
     QString decreaseVolumeName = i18n( "Decrease Volume" );
     decreaseVolumeName += " - " + mixDevice()->readableName() + ", " + mixDevice()->mixer()->readableName();
     b->setText( decreaseVolumeName );
-    #ifdef __GNUC__
+/*    #ifdef __GNUC__
     #warning GLOBAL SHORTCUTS ARE NOW ASSIGNED TO ALL CONTROLS, as enableGlobalShortcut(), has not been committed
-    #endif
+    #endif*/
     if ( ! mixDevice()->mixer()->isDynamic() ) {
         // virtual / dynamic controls won't get shortcuts
         b->setGlobalShortcut(dummyShortcut);  // -<- enableGlobalShortcut() is not there => use workaround
         //   b->enableGlobalShortcut();
-        connect( b, SIGNAL( triggered(bool) ), SLOT( decreaseVolume() ) );
+        connect( b, SIGNAL(triggered(bool)), SLOT(decreaseVolume()) );
     }
 
     b = _mdwPopupActions->addAction( QString("Toggle mute %1").arg( actionSuffix ) );
     QString muteVolumeName = i18n( "Toggle Mute" );
     muteVolumeName += " - " + mixDevice()->readableName() + ", " + mixDevice()->mixer()->readableName();
     b->setText( muteVolumeName );
-    #ifdef __GNUC__
+/*    #ifdef __GNUC__
     #warning GLOBAL SHORTCUTS ARE NOW ASSIGNED TO ALL CONTROLS, as enableGlobalShortcut(), has not been committed
-    #endif
+    #endif*/
     if ( ! mixDevice()->mixer()->isDynamic() ) {
         // virtual / dynamic controls won't get shortcuts
         b->setGlobalShortcut(dummyShortcut);  // -<- enableGlobalShortcut() is not there => use workaround
         //   b->enableGlobalShortcut();
-        connect( b, SIGNAL( triggered(bool) ), SLOT( toggleMuted() ) );
+        connect( b, SIGNAL(triggered(bool)), SLOT(toggleMuted()) );
     }
-    
-    // @todo: The following has been added for an unknown reason. Have to check this. - cesken
-    //        Is there a reason why it is only done for Mute and not Volume Up/down?
-    // 2010-08-20: I found it out: It's a bug ( Bug 209901 -  Volume Up and Volume Down buttons mapped to wrong audio channel)
-    //if (view) view->actionCollection()->addAction( QString("Toggle mute %1").arg( actionSuffix ), b );
+
 }
 
 
@@ -274,9 +283,11 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
     bool wantsPlaybackSliders = includePlayback && ( m_mixdevice->playbackVolume().count() > 0 );
     bool wantsCaptureSliders  = includeCapture && ( m_mixdevice->captureVolume().count() > 0 );
       bool hasVolumeSliders = wantsPlaybackSliders || wantsCaptureSliders;
-      bool bothCaptureANDPlaybackExist = wantsPlaybackSliders && wantsCaptureSliders;
+     // bool bothCaptureANDPlaybackExist = wantsPlaybackSliders && wantsCaptureSliders;
 	
-	// case of vertical sliders:
+      bool wantsMediaControls = ( m_mixdevice->hasMediaNextControl() || m_mixdevice->hasMediaPlayControl() || m_mixdevice->hasMediaPrevControl() );
+
+      // case of vertical sliders:
 	if ( _orientation == Qt::Vertical )
 	{
 		QVBoxLayout *controlLayout = new QVBoxLayout(this);
@@ -327,9 +338,11 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 		if ( hasVolumeSliders )
 		{
 			if ( wantsPlaybackSliders )
-				addSliders( volLayout, 'p', false );
+				addSliders( volLayout, 'p', m_mixdevice->playbackVolume(), m_slidersPlayback);
 			if ( wantsCaptureSliders )
-				addSliders( volLayout, 'c', bothCaptureANDPlaybackExist );
+				addSliders( volLayout, 'c', m_mixdevice->captureVolume() , m_slidersCapture );
+			if ( wantsMediaControls )
+				addMediaControls( volLayout ); // Please note that the addmediaControls() is in the hasVolumeSliders check onyl because it was easier to integrate
 			controlLayout->addSpacing( 3 );
 		} else {
 			controlLayout->addStretch(1);
@@ -341,7 +354,7 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 			m_captureCheckbox = new QCheckBox( i18n("capture") , this);
 			m_captureCheckbox->installEventFilter( this );
 			controlLayout->addWidget( m_captureCheckbox, 0, Qt::AlignHCenter );
-			connect( m_captureCheckbox, SIGNAL( toggled(bool)), this, SLOT( setRecsrc(bool) ) );
+			connect( m_captureCheckbox, SIGNAL(toggled(bool)), this, SLOT(setRecsrc(bool)) );
 			QString muteTip( i18n( "Capture/Uncapture %1", m_mixdevice->readableName() ) );
 			m_captureCheckbox->setToolTip( muteTip );
 		}
@@ -363,7 +376,7 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 
 			controlLayout->addWidget( m_qcb , 0, Qt::AlignHCenter);
 			m_qcb->installEventFilter(this);
-			connect ( m_qcb, SIGNAL( clicked(bool) ), this, SLOT( toggleMuted() ) );
+			connect ( m_qcb, SIGNAL(clicked(bool)), this, SLOT(toggleMuted()) );
 			QString muteTip( i18n( "Mute/Unmute %1", m_mixdevice->readableName() ) );
 			m_qcb->setToolTip( muteTip );
 		}
@@ -387,15 +400,13 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 		row1->addWidget( m_label );
 		row1->setAlignment(m_label, Qt::AlignVCenter);
 
-		//row1->addStretch();
-
 		if ( showCaptureLED && includeCapture && m_mixdevice->captureVolume().hasSwitch() )
 		{
 			m_captureCheckbox = new QCheckBox( i18n("capture") , this);
 			m_captureCheckbox->installEventFilter( this );
 			row1->addWidget( m_captureCheckbox);
 			row1->setAlignment(m_captureCheckbox, Qt::AlignRight);
-			connect( m_captureCheckbox, SIGNAL( toggled(bool)), this, SLOT( setRecsrc(bool) ) );
+			connect( m_captureCheckbox, SIGNAL(toggled(bool)), this, SLOT(setRecsrc(bool)) );
 			QString muteTip( i18n( "Capture/Uncapture %1", m_mixdevice->readableName() ) );
 			m_captureCheckbox->setToolTip( muteTip );
 		}
@@ -430,9 +441,11 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 		if ( hasVolumeSliders )
 		{
 			if ( wantsPlaybackSliders && m_mixdevice->playbackVolume().count() > 0 )
-				addSliders( volLayout, 'p', false );
+				addSliders( volLayout, 'p', m_mixdevice->playbackVolume(), m_slidersPlayback );
 			if ( wantsCaptureSliders && m_mixdevice->captureVolume().count() > 0 )
-				addSliders( volLayout, 'c', bothCaptureANDPlaybackExist );
+				addSliders( volLayout, 'c', m_mixdevice->captureVolume() , m_slidersCapture );
+			if ( wantsMediaControls )
+				addMediaControls( volLayout );
 		}
 		else
 		{
@@ -448,7 +461,7 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 			m_qcb->setIcon( QIcon( loadIcon("audio-volume-muted") ) );
 			row2->addWidget( m_qcb );
 			m_qcb->installEventFilter(this);
-			connect ( m_qcb, SIGNAL( clicked(bool) ), this, SLOT( toggleMuted() ) );
+			connect ( m_qcb, SIGNAL(clicked(bool)), this, SLOT(toggleMuted()) );
 			QString muteTip( i18n( "Mute/Unmute %1", m_mixdevice->readableName() ) );
 			m_qcb->setToolTip( muteTip );
 		}
@@ -464,102 +477,158 @@ void MDWSlider::createWidgets( bool showMuteButton, bool showCaptureLED )
 	layout()->activate(); // Activate it explicitly in KDE3 because of PanelApplet/kicker issues
 }
 
+    void MDWSlider::addMediaControls(QBoxLayout* volLayout)
+    {
+      QBoxLayout *mediaLayout;
+      if ( _orientation == Qt::Vertical )
+	mediaLayout = new QVBoxLayout();
+      else
+	mediaLayout = new QHBoxLayout();
 
-void MDWSlider::addSliders( QBoxLayout *volLayout, char type, bool addLabel)
+      if ( mixDevice()->hasMediaPlayControl())
+      {
+	QToolButton *lbl = addMediaButton("media-skip-backward", mediaLayout);
+	connect(lbl, SIGNAL(clicked(bool)), this, SLOT(mediaPrev(bool)) ); 
+      }
+      if ( mixDevice()->hasMediaPlayControl())
+      {
+	QToolButton *lbl = addMediaButton("media-playback-start", mediaLayout);
+	connect(lbl, SIGNAL(clicked(bool)), this, SLOT(mediaPlay(bool)) ); 
+      }
+      if ( mixDevice()->hasMediaPlayControl())
+      {
+	QToolButton *lbl = addMediaButton("media-skip-forward", mediaLayout);
+	connect(lbl, SIGNAL(clicked(bool)), this, SLOT(mediaNext(bool)) ); 
+      }
+      volLayout->addLayout(mediaLayout);
+    }
+
+
+QToolButton* MDWSlider::addMediaButton(QString iconName, QLayout* layout)
 {
-	Volume* volP;
-	QList<Volume::ChannelID>* ref_slidersChidsP;
-	QList<QWidget *>* ref_slidersP;
+	QToolButton *lbl = new QToolButton(this);
+	lbl->setIconSize(QSize(22,22));
+	lbl->setAutoRaise(true);
+	lbl->setCheckable(false);
+	
+	setIcon(iconName, lbl);
+	layout->addWidget(lbl);
 
-	if ( type == 'c' ) { // capture
-		volP              = &m_mixdevice->captureVolume();
-		ref_slidersChidsP = &_slidersChidsCapture;
-		ref_slidersP      = &m_slidersCapture;
-	}
-	else { // playback
-	  
-		volP              = &m_mixdevice->playbackVolume();
-		ref_slidersChidsP = &_slidersChidsPlayback;
-		ref_slidersP      = &m_slidersPlayback;
-	}
+	return lbl;
+}
 
-	Volume& vol = *volP;
-	QList<Volume::ChannelID>& ref_slidersChids = *ref_slidersChidsP;
-	QList<QWidget *>& ref_sliders = *ref_slidersP;
+void MDWSlider::mediaPrev(bool)
+{
+  mixDevice()->mediaPrev();
+}
 
+void MDWSlider::mediaNext(bool)
+{
+  mixDevice()->mediaNext();
+}
 
-	if (addLabel)
-	{
-		static QString capture = i18n("capture");
-
-		if ( type == 'c' ) { // capture
-			if (_orientation == Qt::Horizontal)
-				m_extraCaptureLabel = new QLabel(capture, this);
-			else
-				m_extraCaptureLabel = new VerticalText(this, capture);
-		}
-		m_extraCaptureLabel->installEventFilter( this );
-		volLayout->addWidget(m_extraCaptureLabel);
-
-	}
+void MDWSlider::mediaPlay(bool)
+{
+  mixDevice()->mediaPlay();
+}
 
+void MDWSlider::addSliders( QBoxLayout *volLayout, char type, Volume& vol, QList<QAbstractSlider *>& ref_sliders)
+{
 	const int minSliderSize = fontMetrics().height() * 10;
-	for ( int i=0; i<= Volume::CHIDMAX; i++ ) {
-		if ( vol._chmask & Volume::_channelMaskEnum[i] ) {
-			Volume::ChannelID chid = Volume::ChannelID(i);
+	long minvol = vol.minVolume();
+	long maxvol = vol.maxVolume();
 
-			long minvol = vol.minVolume();
-			long maxvol = vol.maxVolume();
+	QMap<Volume::ChannelID, VolumeChannel> vols = vol.getVolumes();
 
-			QWidget* slider;
-			if ( m_small ) {
-				slider = new KSmallSlider( minvol, maxvol, (maxvol-minvol)/10, // @todo !! User definable steps
-				                           vol.getVolume( chid ), _orientation, this );
-			} // small
-			else  {
-				QSlider* sliderBig = new QSlider( _orientation, this );
-				slider = sliderBig;
-				sliderBig->setMinimum(0);
-				sliderBig->setMaximum(maxvol);
-				sliderBig->setPageStep(maxvol/10);
-				sliderBig->setValue( maxvol - vol.getVolume( chid ) );
+	foreach (VolumeChannel vc, vols )
+	{
+		//kDebug(67100) << "Add label to " << vc.chid << ": " <<  Volume::ChannelNameReadable[vc.chid];
+		QWidget *subcontrolLabel;
 
-				if ( _orientation == Qt::Vertical ) {
-					sliderBig->setMinimumHeight( minSliderSize );
-				}
-				else {
-					sliderBig->setMinimumWidth( minSliderSize );
-				}
-				if ( ! _pctl->getBackgroundColor().isEmpty() ) {
-				    QString bcolor(_pctl->getBackgroundColor());
-				    QColor qcol(bcolor);
-				    QPalette qpal = sliderBig->palette();
-				    qpal.setColor(QPalette::Window, qcol);
-				    sliderBig->setPalette(qpal);
-				}
-			} // not small
+		QString subcontrolTranslation;
+		if ( type == 'c' ) subcontrolTranslation += i18n("Capture") + " ";
+		subcontrolTranslation += Volume::ChannelNameReadable[vc.chid]; //Volume::getSubcontrolTranslation(chid);
+		subcontrolLabel = createLabel(this, subcontrolTranslation, volLayout, true);
 
-			slider->installEventFilter( this );
-			if ( type == 'p' ) {
-				slider->setToolTip( m_mixdevice->readableName() );
+		QAbstractSlider* slider;
+		if ( m_small )
+		{
+			slider = new KSmallSlider( minvol, maxvol, (maxvol-minvol+1) / Mixer::VOLUME_PAGESTEP_DIVISOR,
+				                           vol.getVolume( vc.chid ), _orientation, this );
+		} // small
+		else  {
+			slider = new VolumeSlider( _orientation, this );
+			slider->setMinimum(minvol);
+			slider->setMaximum(maxvol);
+			slider->setPageStep(maxvol / Mixer::VOLUME_PAGESTEP_DIVISOR);
+			slider->setValue( maxvol - vol.getVolume( vc.chid ) );
+			extraData(slider).setSubcontrolLabel(subcontrolLabel);
+
+			if ( _orientation == Qt::Vertical ) {
+				slider->setMinimumHeight( minSliderSize );
 			}
 			else {
-				QString captureTip( i18n( "%1 (capture)", m_mixdevice->readableName() ) );
-				slider->setToolTip( captureTip );
+				slider->setMinimumWidth( minSliderSize );
 			}
+			if ( ! _pctl->getBackgroundColor().isEmpty() ) {
+				slider->setStyleSheet("QSlider { background-color: " + _pctl->getBackgroundColor() + " }");
+			}
+		} // not small
 
-			if( i>0 && isStereoLinked() ) {
-				// show only one (the first) slider, when the user wants it so
-				slider->hide();
-			}
-			volLayout->addWidget( slider ); // add to layout
-			ref_sliders.append ( slider ); // add to list
-			ref_slidersChids.append(chid);
-			connect( slider, SIGNAL( valueChanged(int) ), SLOT( volumeChange(int) ) );
-		} //if channel is present
+		extraData(slider).setChid(vc.chid);
+		slider->installEventFilter( this );
+		if ( type == 'p' ) {
+			slider->setToolTip( m_mixdevice->readableName() );
+		}
+		else {
+			QString captureTip( i18n( "%1 (capture)", m_mixdevice->readableName() ) );
+			slider->setToolTip( captureTip );
+		}
+
+		volLayout->addWidget( slider ); // add to layout
+		ref_sliders.append ( slider ); // add to list
+		//ref_slidersChids.append(vc.chid);
+		connect( slider, SIGNAL(valueChanged(int)), SLOT(volumeChange(int)) );
+		
 	} // for all channels of this device
 }
 
+
+VolumeSliderExtraData& MDWSlider::extraData(QAbstractSlider *slider)
+{
+  VolumeSlider* sl = qobject_cast<VolumeSlider*>(slider);
+  if ( sl ) return sl->extraData;
+  
+  KSmallSlider* sl2 = qobject_cast<KSmallSlider*>(slider);
+  if ( sl2 ) return sl2->extraData;
+  
+  kError(67100) << "Invalid slider";
+  return MDWSlider::DummVolumeSliderExtraData;
+}
+
+QWidget* MDWSlider::createLabel(QWidget* parent, QString& label, QBoxLayout *layout, bool small)
+{
+  QFont qf;
+  qf.setPointSize(8);
+
+  QWidget* labelWidget;
+	if (_orientation == Qt::Horizontal)
+	{
+		labelWidget = new QLabel(label, parent);
+		if ( small ) ((QLabel*)labelWidget)->setFont(qf);
+	}
+	else {
+		labelWidget = new VerticalText(parent, label);
+		if ( small ) ((VerticalText*)labelWidget)->setFont(qf);
+	}
+	
+	labelWidget->installEventFilter( parent );
+	layout->addWidget(labelWidget);
+
+	return labelWidget;
+}
+
+
 QPixmap MDWSlider::loadIcon( QString filename )
 {
 	return KIconLoader::global()->loadIcon( filename, KIconLoader::Small, KIconLoader::SizeSmallMedium );
@@ -567,12 +636,21 @@ QPixmap MDWSlider::loadIcon( QString filename )
 
 void MDWSlider::setIcon( QString filename )
 {
-	if( !m_iconLabelSimple )
-	{
-		m_iconLabelSimple = new QLabel(this);
-		installEventFilter( m_iconLabelSimple );
-	}
+  setIcon(filename, &m_iconLabelSimple);
+}
 
+void MDWSlider::setIcon( QString filename, QLabel** label )
+{
+	if( (*label) == 0 )
+	{
+		*label = new QLabel(this);
+		installEventFilter( *label );
+	}
+	setIcon(filename, *label);
+}
+
+void MDWSlider::setIcon( QString filename, QWidget* label )
+{
 	QPixmap miniDevPM = loadIcon( filename );
 	if ( !miniDevPM.isNull() )
 	{
@@ -581,16 +659,31 @@ void MDWSlider::setIcon( QString filename )
 			// scale icon
 			QMatrix t;
 			t = t.scale( 10.0/miniDevPM.width(), 10.0/miniDevPM.height() );
-			m_iconLabelSimple->setPixmap( miniDevPM.transformed( t ) );
-			m_iconLabelSimple->resize( 10, 10 );
+			miniDevPM = miniDevPM.transformed( t );
+			label->resize( 10, 10 );
 		} // small size
+		label->setMinimumSize(22,22);
+		
+		QLabel* lbl = qobject_cast<QLabel*>(label);
+		if ( lbl != 0 )
+		{
+		  lbl->setPixmap( miniDevPM );
+		  lbl->setAlignment(Qt::AlignHCenter | Qt::AlignCenter);
+		} // QLabel
 		else
 		{
-			m_iconLabelSimple->setPixmap( miniDevPM );
-		} // normal size
-
-		m_iconLabelSimple->setMinimumSize(22,22);
-		m_iconLabelSimple->setAlignment(Qt::AlignHCenter | Qt::AlignCenter);
+		  QToolButton* tbt = qobject_cast<QToolButton*>(label);
+		  if ( tbt != 0 )
+		  {
+/*		  QIcon ic(miniDevPM);
+		  lbl->setIcon(ic);*/
+		    tbt->setIcon( miniDevPM );
+//		    tbt->setAlignment(Qt::AlignHCenter | Qt::AlignCenter);
+		  } // QToolButton 
+		  else {
+		    kError() << "Invalid widget type ... cannot set icon";
+		  }
+		}
 	}
 	else
 	{
@@ -613,53 +706,48 @@ void
 MDWSlider::setStereoLinked(bool value)
 {
 	m_linked = value;
-	if (m_slidersPlayback.count() != 0) setStereoLinkedInternal(m_slidersPlayback);
-	if (m_slidersCapture.count() != 0) setStereoLinkedInternal(m_slidersCapture);
+
+	int overallSlidersToShow = 0;
+	if ( ! m_slidersPlayback.isEmpty() ) overallSlidersToShow += ( m_linked ? 1 : m_slidersPlayback.count() );
+	if ( ! m_slidersCapture.isEmpty()  ) overallSlidersToShow += ( m_linked ? 1 : m_slidersCapture.count() );
+
+	bool showSubcontrolLabels = (overallSlidersToShow >= 2);
+	setStereoLinkedInternal(m_slidersPlayback, showSubcontrolLabels);
+	setStereoLinkedInternal(m_slidersCapture  , showSubcontrolLabels);
 	update(); // Call update(), so that the sliders can adjust EITHER to the individual values OR the average value.
 }
 
 void
-MDWSlider::setStereoLinkedInternal(QList<QWidget *>& ref_sliders)
+MDWSlider::setStereoLinkedInternal(QList<QAbstractSlider *>& ref_sliders, bool showSubcontrolLabels)
 {
-	QWidget *slider = ref_sliders[0];
+	if ( ref_sliders.isEmpty()) return;
+	// kDebug() << "m_linked=" << m_linked << "showSubcontrolLabels" << showSubcontrolLabels;
 
-	/***********************************************************
-	   Remember value of first slider, so that it can be copied
-	   to the other sliders.
-	***********************************************************/
-	int firstSliderValue = 0;
-	bool firstSliderValueValid = false;
-	if ( ::qobject_cast<QSlider*>(slider) ) {
-		QSlider *sld = static_cast<QSlider*>(slider);
-		firstSliderValue = sld->value();
-		firstSliderValueValid = true;
-	}
-	else if ( ::qobject_cast<KSmallSlider*>(slider) )  {
-		KSmallSlider *sld = static_cast<KSmallSlider*>(slider);
-		firstSliderValue = sld->value();
-		firstSliderValueValid = true;
+	bool first = true;
+	foreach ( QAbstractSlider* slider1, ref_sliders )
+	{
+		slider1->setVisible(!m_linked || first); // One slider (the 1st) is always shown
+		extraData(slider1).getSubcontrolLabel()->setVisible(!m_linked && showSubcontrolLabels); // (*)
+		first = false;
+		/* (*) cesken: I have excluded the "|| first" check because the text would not be nice:
+		 *     It would be "Left" or "Capture Left", while it should be "Playback" and "Capture" in the "linked" case.
+		 *
+		 *     But the only affected situation is when we have playback AND capture on the same control, where we show no label.
+		 *     It would be nice to put at least a "Capture" label on the capture subcontrol instead.
+		 *     To achieve this we would need to exchange the Text on the first capture subcontrol dynamically. This can
+		 *     be done, but I'll leave this open for now.
+		 */
 	}
 
-	for( int i=1; i<ref_sliders.count(); ++i ) {
-		slider = ref_sliders[i];
-		if ( slider == 0 ) {
-			continue;
-		}
-		if ( m_linked ) {
-			slider->hide();
-		}
-		else {
-			slider->show();
-		}
-	}
+
 
 	// Redo the tickmarks to last slider in the slider list.
 	// The implementation is not obvious, so lets explain:
 	// We ALWAYS have tickmarks on the LAST slider. Sometimes the slider is not shown, and then we just don't bother.
 	// a) So, if the last slider has tickmarks, we can always call setTicks( true ).
 	// b) if the last slider has NO tickmarks, there ae no tickmarks at all, and we don't need to redo the tickmarks.
-	slider = ref_sliders.last();
-	if( slider && ( static_cast<QSlider *>(slider)->tickPosition() != QSlider::NoTicks) )
+	QSlider* slider = qobject_cast<QSlider*>( ref_sliders.last() );
+	if( slider && slider->tickPosition() != QSlider::NoTicks) 
 		setTicks( true );
 
 }
@@ -668,21 +756,9 @@ MDWSlider::setStereoLinkedInternal(QList<QWidget *>& ref_sliders)
 void
 MDWSlider::setLabeled(bool value)
 {
-	if ( m_label == 0  && m_extraCaptureLabel == 0 )
-		return;
-
-	if (value ) {
-		if ( m_label != 0) m_label->show();
-		if ( m_extraCaptureLabel != 0) m_extraCaptureLabel->show();
-		if ( m_muteText != 0) m_muteText->show();
-		if ( m_captureText != 0) m_captureText->show();
-	}
-	else {
-		if ( m_label != 0) m_label->hide();
-		if ( m_extraCaptureLabel != 0) m_extraCaptureLabel->hide();
-		if ( m_muteText != 0) m_muteText->hide();
-		if ( m_captureText != 0) m_captureText->hide();
-	}
+	if ( m_label != 0) m_label->setVisible(value);
+	if ( m_muteText != 0) m_muteText->setVisible(value);
+	if ( m_captureText != 0) m_captureText->setVisible(value);
 	layout()->activate();
 }
 
@@ -693,29 +769,33 @@ MDWSlider::setTicks( bool value )
 	if (m_slidersCapture.count() != 0) setTicksInternal(m_slidersCapture, value);
 }
 
-void
-MDWSlider::setTicksInternal(QList<QWidget *>& ref_sliders, bool ticks)
+/**
+ * Enables or disables tickmarks
+ * Please note that always only the first and last slider has tickmarks.
+ * 
+ */
+void MDWSlider::setTicksInternal(QList<QAbstractSlider *>& ref_sliders, bool ticks)
 {
-	QWidget* slider = ref_sliders[0];
-
-	if ( slider->inherits( "QSlider" ) )
+	VolumeSlider* slider = qobject_cast<VolumeSlider*>( ref_sliders[0]);
+	if (slider == 0 ) return; // Ticks are only in VolumeSlider, but not in KSmallslider
+	
+	if( ticks )
 	{
-		if( ticks )
-			if( isStereoLinked() )
-				static_cast<QSlider *>(slider)->setTickPosition( QSlider::TicksRight );
-			else
-			{
-				static_cast<QSlider *>(slider)->setTickPosition( QSlider::NoTicks );
-				slider = ref_sliders.last();
-				static_cast<QSlider *>(slider)->setTickPosition( QSlider::TicksLeft );
-			}
+		if( isStereoLinked() )
+			slider->setTickPosition( QSlider::TicksRight );
 		else
 		{
-			static_cast<QSlider *>(slider)->setTickPosition( QSlider::NoTicks );
-			slider = ref_sliders.last();
-			static_cast<QSlider *>(slider)->setTickPosition( QSlider::NoTicks );
+			slider->setTickPosition( QSlider::NoTicks );
+			slider = qobject_cast<VolumeSlider*>(ref_sliders.last());
+			slider->setTickPosition( QSlider::TicksLeft );
 		}
 	}
+	else
+	{
+		slider->setTickPosition( QSlider::NoTicks );
+		slider = qobject_cast<VolumeSlider*>(ref_sliders.last());
+		slider->setTickPosition( QSlider::NoTicks );
+	}
 }
 
 void
@@ -767,47 +847,36 @@ MDWSlider::setMutedColors( QColor high, QColor low, QColor back )
 /** This slot is called, when a user has changed the volume via the KMix Slider. */
 void MDWSlider::volumeChange( int )
 {
-	if (m_slidersPlayback.count() > 0) volumeChangeInternal(m_mixdevice->playbackVolume(), _slidersChidsPlayback, m_slidersPlayback);
-	if (m_slidersCapture.count()  > 0) volumeChangeInternal(m_mixdevice->captureVolume(), _slidersChidsCapture, m_slidersCapture);
+//  if ( mixDevice()->id() == "Headphone:0" )
+//  {
+//    kDebug(67100) << "headphone bug";
+//  }
+	if (m_slidersPlayback.count() > 0) volumeChangeInternal(m_mixdevice->playbackVolume(), m_slidersPlayback);
+	if (m_slidersCapture.count()  > 0) volumeChangeInternal(m_mixdevice->captureVolume(), m_slidersCapture);
 	m_mixdevice->mixer()->commitVolumeChange(m_mixdevice);
 }
 
-void MDWSlider::volumeChangeInternal( Volume& vol, QList<Volume::ChannelID>& ref_slidersChids, QList<QWidget *>& ref_sliders  )
+void MDWSlider::volumeChangeInternal( Volume& vol, QList<QAbstractSlider *>& ref_sliders  )
 {
 
 	// --- Step 2: Change the volumes directly in the Volume object to reflect the Sliders ---
 	if ( isStereoLinked() )
 	{
-		long firstVolume = 0;
-		if ( ref_sliders.first()->inherits( "KSmallSlider" ) )
-		{
-			KSmallSlider *slider = dynamic_cast<KSmallSlider *>( ref_sliders.first() );
-			if (slider != 0 ) firstVolume = slider->value();
-		}
-		else
-		{
-			QSlider *slider = dynamic_cast<QSlider *>( ref_sliders.first() );
-			if (slider != 0 ) firstVolume = slider->value();
-		}
+		QAbstractSlider* firstSlider = ref_sliders.first();
+		long firstVolume = firstSlider->value();
+		//kDebug(67100) << "firstVolume=" <<firstVolume;
 		vol.setAllVolumes(firstVolume);
 	} // stereoLinked()
 
 	else {
-		QList<Volume::ChannelID>::Iterator it = ref_slidersChids.begin();
-		for( int i=0; i<ref_sliders.count(); i++, ++it ) {
-			Volume::ChannelID chid = *it;
-			QWidget *sliderWidget = ref_sliders[i];
 
-			if ( sliderWidget->inherits( "KSmallSlider" ) )
-			{
-				KSmallSlider *slider = dynamic_cast<KSmallSlider *>(sliderWidget);
-				if (slider) vol.setVolume( chid, slider->value() );
-			}
-			else
-			{
-				QSlider *slider = dynamic_cast<QSlider *>(sliderWidget);
-				if (slider) vol.setVolume( chid, slider->value() );
-			}
+	  	QAbstractSlider* firstSlider = ref_sliders.first();
+		long firstVolume = firstSlider->value();
+		//kDebug(67100) << "firstVolume=" <<firstVolume;
+	  
+		for( int i=0; i<ref_sliders.count(); i++ ) {
+			QAbstractSlider *sliderWidget = ref_sliders[i];
+			vol.setVolume( extraData(sliderWidget).getChid() ,sliderWidget->value());
 		} // iterate over all sliders
 	} // !stereoLinked()
 
@@ -870,24 +939,34 @@ void MDWSlider::setDisabled( bool value )
     associated KAction like the context menu.
  */
 void MDWSlider::increaseVolume()
+{
+  increaseOrDecreaseVolume(false);
+}
+
+void MDWSlider::increaseOrDecreaseVolume(bool decrease)
 {
 	Volume& volP = m_mixdevice->playbackVolume();
-	long inc = volP.maxVolume() / 20;
-	if ( inc == 0 )
-		inc = 1;
-	for ( int i = 0; i < volP.count(); i++ ) {
-		long newVal = (volP[i]) + inc;
-		volP.setVolume( (Volume::ChannelID)i, newVal < volP.maxVolume() ? newVal : volP.maxVolume() );
+	long inc = volP.volumeSpan() / Mixer::VOLUME_STEP_DIVISOR;
+	if ( inc == 0 )	inc = 1;
+	if ( decrease ) inc *= -1;
+//	if ( mixDevice()->id() == "Headphone:0" )
+//	  debugMe =true;
+//	if (debugMe)
+//	  kDebug(67100) << ( decrease ? "decrease by " : "increase by " ) << inc ;
+	if ( !decrease && m_mixdevice->isMuted())
+	{   // increasing form muted state: unmute and start with a low volume level
+	    m_mixdevice->setMuted(false);
+	    volP.setAllVolumes(inc);
 	}
+	else
+	    volP.changeAllVolumes(inc);
 
 	Volume& volC = m_mixdevice->captureVolume();
-	inc = volC.maxVolume() / 20;
-	if ( inc == 0 )
-		inc = 1;
-	for ( int i = 0; i < volC.count(); i++ ) {
-		long newVal = (volC[i]) + inc;
-		volC.setVolume( (Volume::ChannelID)i, newVal < volC.maxVolume() ? newVal : volC.maxVolume() );
-	}
+	inc = volC.volumeSpan() / Mixer::VOLUME_STEP_DIVISOR;
+	if ( inc == 0 ) inc = 1;
+	if ( decrease ) inc *= -1;
+	volC.changeAllVolumes(inc);
+
 	m_mixdevice->mixer()->commitVolumeChange(m_mixdevice);
 }
 
@@ -897,24 +976,7 @@ void MDWSlider::increaseVolume()
  */
 void MDWSlider::decreaseVolume()
 {
-	Volume& volP = m_mixdevice->playbackVolume();
-	long inc = volP.maxVolume() / 20;
-	if ( inc == 0 )
-		inc = 1;
-	for ( int i = 0; i < volP.count(); i++ ) {
-		long newVal = (volP[i]) - inc;
-		volP.setVolume( (Volume::ChannelID)i, newVal > 0 ? newVal : 0 );
-	}
-
-	Volume& volC = m_mixdevice->captureVolume();
-	inc = volC.maxVolume() / 20;
-	if ( inc == 0 )
-		inc = 1;
-	for ( int i = 0; i < volC.count(); i++ ) {
-		long newVal = (volC[i]) - inc;
-		volC.setVolume( (Volume::ChannelID)i, newVal > 0 ? newVal : 0 );
-	}
-	m_mixdevice->mixer()->commitVolumeChange(m_mixdevice);
+  increaseOrDecreaseVolume(true);
 }
 
 
@@ -934,10 +996,11 @@ void MDWSlider::moveStream(QString destId)
  */
 void MDWSlider::update()
 {
+
 	if ( m_slidersPlayback.count() != 0 || m_mixdevice->playbackVolume().hasSwitch() )
-		updateInternal(m_mixdevice->playbackVolume(), m_slidersPlayback, _slidersChidsPlayback);
+		updateInternal(m_mixdevice->playbackVolume(), m_slidersPlayback, m_mixdevice->isMuted() );
 	if ( m_slidersCapture.count()  != 0 || m_mixdevice->captureVolume().hasSwitch() )
-		updateInternal(m_mixdevice->captureVolume(), m_slidersCapture, _slidersChidsCapture );
+		updateInternal(m_mixdevice->captureVolume(), m_slidersCapture, m_mixdevice->isNotRecSource() );
 	if (m_label) {
 		QLabel *l;
 		VerticalText *v;
@@ -948,37 +1011,23 @@ void MDWSlider::update()
 	}
 }
 
-void MDWSlider::updateInternal(Volume& vol, QList<QWidget *>& ref_sliders, QList<Volume::ChannelID>& ref_slidersChids)
+void MDWSlider::updateInternal(Volume& vol, QList<QAbstractSlider *>& ref_sliders, bool muted)
 {
-	// update volumes
-	long useVolume = vol.getAvgVolume( Volume::MMAIN );
-
-
-	QList<Volume::ChannelID>::Iterator it = ref_slidersChids.begin();
-	for( int i=0; i<ref_sliders.count(); i++, ++it ) {
-		if( !isStereoLinked() ) {
-			Volume::ChannelID chid = *it;
-			useVolume = vol[chid];
-		}
-		QWidget *slider = ref_sliders.at( i );
+	for( int i=0; i<ref_sliders.count(); i++ ) {
+		QAbstractSlider *slider = ref_sliders.at( i );
+		Volume::ChannelID chid = extraData(slider).getChid();
+		long useVolume;
+		if ( muted )
+		    useVolume = 0;
+        else
+            useVolume = vol.getVolume(chid);
 
 		slider->blockSignals( true );
-
+		slider->setValue( useVolume );
 		if ( slider->inherits( "KSmallSlider" ) )
 		{
-			KSmallSlider *smallSlider = dynamic_cast<KSmallSlider *>(slider);
-			if (smallSlider) {
-				smallSlider->setValue( useVolume );
-				smallSlider->setGray( m_mixdevice->isMuted() );
-			}
+			((KSmallSlider*)slider)->setGray( m_mixdevice->isMuted() );
 		}
-		else
-		{
-			QSlider *bigSlider = dynamic_cast<QSlider *>(slider);
-			if (bigSlider)
-				bigSlider->setValue( useVolume );
-		}
-
 		slider->blockSignals( false );
 	} // for all sliders
 
@@ -994,12 +1043,6 @@ void MDWSlider::updateInternal(Volume& vol, QList<QWidget *>& ref_sliders, QList
 		m_qcb->blockSignals( false );
 	}
 
-	// update recsrc
-//	if( m_captureLED ) {
-//		m_captureLED->blockSignals( true );
-//		m_captureLED->setState( m_mixdevice->isRecSource() ? KLed::On : KLed::Off );
-//		m_captureLED->blockSignals( false );
-//	}
 	if( m_captureCheckbox ) {
 		m_captureCheckbox->blockSignals( true );
 		m_captureCheckbox->setChecked( m_mixdevice->isRecSource() );
@@ -1100,7 +1143,8 @@ void MDWSlider::showMoveMenu()
  */
 bool MDWSlider::eventFilter( QObject* obj, QEvent* e )
 {
-	if (e->type() == QEvent::MouseButtonPress) {
+	QEvent::Type eventType = e->type();
+	if (eventType == QEvent::MouseButtonPress) {
 		QMouseEvent *qme = static_cast<QMouseEvent*>(e);
 		if (qme->button() == Qt::RightButton) {
 			showContextMenu();
@@ -1108,10 +1152,16 @@ bool MDWSlider::eventFilter( QObject* obj, QEvent* e )
 		}
 	}
 	// Attention: We don't filter WheelEvents for KSmallSlider, because it handles WheelEvents itself
-	else if ( (e->type() == QEvent::Wheel)
-	         && strcmp(obj->metaObject()->className(),"KSmallSlider") != 0 )  {
+	else if ( (eventType == QEvent::Wheel) )
+//	         && strcmp(obj->metaObject()->className(),"KSmallSlider") != 0 )  {  // Remove the KSmallSlider check. If KSmallSlider comes back, use a cheaper type check - e.g. a boolean value.
+	{
 		QWheelEvent *qwe = static_cast<QWheelEvent*>(e);
-		if (qwe->delta() > 0) {
+
+		bool increase = (qwe->delta() > 0);
+		if (qwe->orientation() == Qt::Horizontal) // Reverse horizontal scroll: bko228780 
+			increase = !increase;
+
+		if (increase) {
 			increaseVolume();
 		}
 		else {
